@@ -36,8 +36,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <QScroller>
 #include <QTimeLine>
 #include <QTimer>
+#include <QWindow>
 
 // KF
+#include <KDirLister>
 #include <KDirModel>
 #include <KIconLoader>
 #include <KPixmapSequence>
@@ -54,6 +56,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "urlutils.h"
 #include <lib/gvdebug.h>
 #include <lib/scrollerutils.h>
+#include <lib/semanticinfo/sorteddirmodel.h>
 #include <lib/thumbnailprovider/thumbnailprovider.h>
 #include <lib/touch/touch.h>
 
@@ -170,15 +173,65 @@ using ThumbnailForUrl = QHash<QUrl, Thumbnail>;
 using UrlQueue = QQueue<QUrl>;
 using PersistentModelIndexSet = QSet<QPersistentModelIndex>;
 
+class WindowScaleWatcher : public QObject
+{
+    Q_OBJECT
+public:
+    void setWindow(QWindow *window)
+    {
+        if (mWindow == window) {
+            return;
+        }
+        if (mWindow) {
+            mWindow->removeEventFilter(this);
+        }
+        mWindow = window;
+        if (mWindow) {
+            mWindow->installEventFilter(this);
+            updateDevicePixelRatio();
+        }
+    }
+
+protected:
+    bool eventFilter(QObject *watched, QEvent *event) override
+    {
+        if (event->type() == QEvent::DevicePixelRatioChange) {
+            updateDevicePixelRatio();
+        }
+        return QObject::eventFilter(watched, event);
+    }
+
+private:
+    void updateDevicePixelRatio()
+    {
+        if (mWindow->devicePixelRatio() != mLastDevicePixelRatio) {
+            Q_EMIT scaleChanged();
+            mLastDevicePixelRatio = mWindow->devicePixelRatio();
+        }
+    }
+
+Q_SIGNALS:
+    void scaleChanged();
+
+private:
+    QPointer<QWindow> mWindow;
+    // we don't know what dpr the widget was using
+    // before getting a window as it comes from the primary screen
+    // always trigger on the first set
+    qreal mLastDevicePixelRatio = -1;
+};
+
 struct ThumbnailViewPrivate {
     ThumbnailView *q;
     ThumbnailView::ThumbnailScaleMode mScaleMode;
     QSize mThumbnailSize;
+    int mThumbnailLogicalWidth = 0;
     qreal mThumbnailAspectRatio;
     AbstractDocumentInfoProvider *mDocumentInfoProvider;
     AbstractThumbnailViewHelper *mThumbnailViewHelper;
     ThumbnailForUrl mThumbnailForUrl;
     QTimer mScheduledThumbnailGenerationTimer;
+    WindowScaleWatcher mWindowScaleWatcher;
 
     UrlQueue mSmoothThumbnailQueue;
     QTimer mSmoothThumbnailTimer;
@@ -195,6 +248,8 @@ struct ThumbnailViewPrivate {
     QScroller *mScroller;
     Touch *mTouch;
 
+    bool loading = false;
+
     void setupBusyAnimation()
     {
         mBusySequence = KPixmapSequenceLoader::load(QStringLiteral("process-working"), 22);
@@ -328,6 +383,10 @@ ThumbnailView::ThumbnailView(QWidget *parent)
 
     connect(this, &ThumbnailView::activated, this, &ThumbnailView::emitIndexActivatedIfNoModifiers);
 
+    connect(&d->mWindowScaleWatcher, &WindowScaleWatcher::scaleChanged, this, [this]() {
+        setThumbnailWidth(d->mThumbnailLogicalWidth);
+    });
+
     d->mScroller = ScrollerUtils::setQScroller(this->viewport());
     d->mTouch = new Touch(viewport());
     connect(d->mTouch, &Touch::twoFingerTapTriggered, this, &ThumbnailView::showContextMenu);
@@ -364,6 +423,11 @@ void ThumbnailView::setModel(QAbstractItemModel *newModel)
 {
     if (model()) {
         disconnect(model(), nullptr, this, nullptr);
+
+        const auto sortedModel = qobject_cast<SortedDirModel *>(newModel);
+        if (sortedModel) {
+            sortedModel->dirLister()->disconnect(this);
+        }
     }
     QListView::setModel(newModel);
 
@@ -373,6 +437,18 @@ void ThumbnailView::setModel(QAbstractItemModel *newModel)
             Q_EMIT rowsRemovedSignal(index, first, last);
         }
     });
+
+    const auto sortedModel = qobject_cast<SortedDirModel *>(newModel);
+    if (sortedModel) {
+        connect(sortedModel->dirLister(), &KDirLister::started, this, [this]() {
+            d->loading = true;
+        });
+
+        connect(sortedModel->dirLister(), &KDirLister::listingDirCompleted, this, [this]() {
+            d->loading = false;
+            d->scheduleThumbnailGeneration();
+        });
+    }
 }
 
 void ThumbnailView::setThumbnailProvider(ThumbnailProvider *thumbnailProvider)
@@ -403,7 +479,7 @@ void ThumbnailView::updateThumbnailSize()
     pix.fill(Qt::transparent);
     QPainter painter(&pix);
     painter.setOpacity(0.5);
-    painter.drawPixmap((value.width() - icon.width()) / 2, (value.height() - icon.height()) / 2, icon);
+    style()->drawItemPixmap(&painter, QRect(QPoint(), pix.deviceIndependentSize().toSize()), Qt::AlignCenter, icon);
     painter.end();
     d->mWaitingThumbnail = pix;
     d->mWaitingThumbnail.setDevicePixelRatio(dpr);
@@ -428,6 +504,7 @@ void ThumbnailView::updateThumbnailSize()
 
 void ThumbnailView::setThumbnailWidth(int width)
 {
+    d->mThumbnailLogicalWidth = width;
     const auto dpr = devicePixelRatioF();
     const qreal newWidthF = width * dpr;
     const int newWidth = qRound(newWidthF);
@@ -835,6 +912,8 @@ void ThumbnailView::resizeEvent(QResizeEvent *event)
 
 void ThumbnailView::showEvent(QShowEvent *event)
 {
+    d->mWindowScaleWatcher.setWindow(window()->windowHandle());
+
     QListView::showEvent(event);
     d->scheduleThumbnailGeneration();
     QTimer::singleShot(0, this, &ThumbnailView::scrollToSelectedIndex);
@@ -890,7 +969,7 @@ void ThumbnailView::scrollContentsBy(int dx, int dy)
 
 void ThumbnailView::generateThumbnailsForItems()
 {
-    if (!isVisible() || !model()) {
+    if (!isVisible() || !model() || d->loading) {
         return;
     }
     const QRect visibleRect = viewport()->rect();
@@ -929,7 +1008,7 @@ void ThumbnailView::generateThumbnailsForItems()
         // Compute distance
         int distance;
         const QRect itemRect = visualRect(index);
-        if (itemRect.isValid()) {
+        if (itemRect.intersected(visibleRect).isValid()) {
             // Item is visible, order thumbnails from left to right, top to bottom
             // Distance is computed so that it is between 0 and visibleSurface
             distance = itemRect.top() * visibleRect.width() + itemRect.left();
@@ -1084,4 +1163,4 @@ void ThumbnailView::setCreateThumbnailsForRemoteUrls(bool createRemoteThumbs)
 
 } // namespace
 
-#include "moc_thumbnailview.cpp"
+#include "thumbnailview.moc"
